// 3-D electromagnetic wave scattering based on E-field Helmholtz
// equation and truncated Silver-Muller condition.

// To run: getdp wave_SM_HH.pro -pre wave_SilverMuller -cal -pos CExtSurf -msh sphere.msh
// Calculates the extinction cross-section.

Include "param.pro"

Group {
  // Region around the scatterer.
  VolumePropa = Region[VolumePropaIndex];

  // The scattering object.
  VolumeObject = Region[VolumeObjectIndex];

  // Total solution domain.
  VolumeTotal = Region[{VolumePropa,VolumeObject}];
  
  // Outer surface.
  SurfaceSilverMuller = Region[SurfaceSilverMullerIndex];

  // All surfaces.
  SurfaceTotal = Region[{SurfaceSilverMuller}];

  // All domains and surfaces.
  VolSurfTot = Region[{VolumePropa,VolumeObject,SurfaceSilverMuller}];
}

Function {
  // Relative wavelength.
  Lambda = 1200e-9/Scale;

  // Real and imaginary parts of refractive index (assuming time exp(-iwt)).
  ri1List = ListFromFile["aujc_re.ref"];
  ri2List = ListFromFile["aujc_im.ref"];

  // Refractive index as a function of wavelength.
  // Takes as input the relative wavelength.
  ri[] = Complex[InterpolationLinear[$1*Scale]{List[ri1List]},
		 InterpolationLinear[$1*Scale]{List[ri2List]}];
		 
  // Frequency in Hz. C is 3e8 m/s.
  Freq = C/Lambda;

  c[] = C;
  omega = 2*Pi*Freq;
  k0 = 2*Pi/Lambda;
  I[] = Complex[0,1];

  // Physical constants of the media.
  eps[VolumePropa] = eps0;
  eps[VolumeObject] = eps0*(ri[Lambda]^2);
  eps[SurfaceSilverMuller] = eps0; // Is this needed?

  mu[VolumePropa] = mu0;
  mu[VolumeObject] = mu0;
  mu[SurfaceSilverMuller] = mu0; // Is this needed?

  k[VolumePropa] = k0;
  k[VolumeObject] = k0*ri[Lambda];
  k[SurfaceSilverMuller] = k0;

  // Incident wave: plane-wave propagating along z-direction
  // and polarized in x-direction.
  kv[VolSurfTot] = k0*Vector[0,0,1];
  phase[VolSurfTot] = CompX[kv[]]*X[] + CompY[kv[]]*Y[] + CompZ[kv[]]*Z[];
  E0[VolSurfTot] = Vector[1,0,0]*Exp[ I[]*phase[] ];
  H0[VolSurfTot] = Cross[kv[], E0[]]/(omega*mu0);
  
  // Excitation function.
  g[VolSurfTot] = I[]*(omega*mu0*Cross[ H0[],Normal[] ] - k0*E0[]);

  // SaveFct[] indicate if we have to save the solution.
  SaveFct[] = 1;
}

Jacobian {
  { Name JVol ;
    Case {
      { Region VolumeTotal ; Jacobian Vol ; }
    }
  }
  { Name JSur ;
    Case {
      { Region SurfaceTotal ; Jacobian Sur ; }
    }
  }
}

Integration {
  { Name I1 ;
    Case {
      { Type Gauss ;
        Case {
          { GeoElement Point       ; NumberOfPoints  1 ; }
          { GeoElement Line        ; NumberOfPoints  2 ; }
          { GeoElement Triangle    ; NumberOfPoints  3 ; }
          { GeoElement Quadrangle  ; NumberOfPoints  9 ; }
          { GeoElement Tetrahedron ; NumberOfPoints  15 ; }
          { GeoElement Hexahedron  ; NumberOfPoints  13 ; }
          { GeoElement Prism       ; NumberOfPoints  13 ; }
        }
      }
    }
  }
}

FunctionSpace {
  //Electric field
  { Name H_curl_e; Type Form1;
    BasisFunction {
      { Name se; NameOfCoef ee; Function BF_Edge;
        Support VolSurfTot ; Entity EdgesOf[All]; }
    }
  }
}

Formulation {
  { Name wave_SilverMuller; Type FemEquation;
    Quantity {
      { Name E; Type Local; NameOfSpace H_curl_e; }
    }
    Equation {
      Galerkin { [Dof{Curl E}, {Curl E} ];
	In VolumeTotal; Integration I1; Jacobian JVol; }

      Galerkin { [-(k[]^2)*Dof{E}, {E} ];
	In VolumeTotal; Integration I1; Jacobian JVol; }

      Galerkin { [-I[]*k0*Dof{E}, {E}];
	In SurfaceSilverMuller; Integration I1; Jacobian JSur; }

	// Excitation over the spherical surface.
      Galerkin { [g[] , {E}];
	In SurfaceSilverMuller; Integration I1; Jacobian JSur; }
    }
  }
}

Resolution {
  { Name wave_SilverMuller;
    System {
      { Name A; NameOfFormulation wave_SilverMuller;
        Type ComplexValue; Frequency Freq; }
    }
    Operation { Generate A ; /*Print A ;*/ Solve A ; SaveSolution A ; }
  }
}

PostProcessing {
  { Name wave_SilverMuller; NameOfFormulation wave_SilverMuller; NameOfSystem A;
    Quantity {
      // Extinction cross-section.
      { Name CExt;
	Value{
	  Integral { [0.5*Scale^2*Re[Normal[]*( Cross[E0[], Conj[{Curl E}]]/(I[]*omega*mu0) + Cross[{E}, Conj[H0[]]] )]] ; In SurfaceSilverMuller; Jacobian JSur; Integration I1;}	}
      }
      { Name E;
	Value{
	  Local{ [ {E}] ; In VolumeTotal; Jacobian JVol; }
	}
      }
    }
  }
}

PostOperation {
  {Name CExtSurf; NameOfPostProcessing wave_SilverMuller;
    Operation {
      Print[ CExt[SurfaceSilverMuller], OnGlobal, Format Table, File >> "cextsur.dat"] ;
    }
  }
  {Name Exy; NameOfPostProcessing wave_SilverMuller;
    Operation {
      Print[ E, OnGrid {$A, $B, 0} {-2:2:0.1, -2:2:0.1,0}, File "Exy.pos", HarmonicToTime 20] ;
  }
 }
}